// checksum 0xfd0b version 0x30001
/*
  This file was generated by the Mobile Qt Application wizard of Qt Creator.
  MainWindow is a convenience class containing mobile device specific code
  such as screen orientation handling.
  It is recommended not to modify this file, since newer versions of Qt Creator
  may offer an updated version of it.
*/

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFileDialog>
#include <QScopedPointer>
#include <QtDebug>

#include <TelepathyQt4/AccountFactory>
#include <TelepathyQt4/PendingOperation>
#include <TelepathyQt4/PendingReady>

#include "client.h"
#include "send-dialog.h"


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // We need Account::FeatureCapabilities to filter file transfer accounts
    Tp::Features accountFeatures = Tp::Features()
            << Tp::Account::FeatureCore
            << Tp::Account::FeatureCapabilities;

    Tp::AccountFactoryPtr accountFactory =
            Tp::AccountFactory::create(QDBusConnection::sessionBus(),
                accountFeatures);

    accountManager = Tp::AccountManager::create(accountFactory);
    connect(accountManager->becomeReady(),
            SIGNAL(finished(Tp::PendingOperation*)),
            SLOT(onAccountManagerReady(Tp::PendingOperation*)));

    clientRegistrar =
            Tp::ClientRegistrar::create(accountManager);

    Tp::SharedPtr<FileTransferClient> client(new FileTransferClient());
    clientRegistrar->registerClient(Tp::AbstractClientPtr::staticCast(client),
                                    "file-transfer-demo");
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::onAccountManagerReady(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qWarning() << "Account manager cannot become ready:" <<
            op->errorName() << "-" << op->errorMessage();
        return;
    }

    // Accounts are now ready and can be filtered
    accountSet = accountManager->fileTransferAccounts();

    // Listen to account list changes
    connect(accountSet.data(), SIGNAL(accountAdded(const Tp::AccountPtr&)),
            SLOT(onAccountAdded(const Tp::AccountPtr&)));
    connect(accountSet.data(), SIGNAL(accountRemoved(const Tp::AccountPtr&)),
            SLOT(onAccountRemoved(const Tp::AccountPtr&)));

    // Populate the account selector
    // TODO: it would be nice to have the accounts sorted by name in the model
    QComboBox *selector = ui->accountSelector;
    foreach (const Tp::AccountPtr &account, accountSet->accounts()) {
        selector->addItem(account->displayName(), account->objectPath());
        // TODO: set FT capabilities for accounts' connections
    }
}

void MainWindow::onAccountAdded(const Tp::AccountPtr &account)
{
    QComboBox *selector = ui->accountSelector;
    selector->addItem(account->displayName(), account->objectPath());
}

void MainWindow::onAccountRemoved(const Tp::AccountPtr &account)
{
    QComboBox *selector = ui->accountSelector;
    selector->removeItem(selector->findData(account->objectPath()));
}

void MainWindow::on_fileBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this,
            "Select file for sending");
    ui->fileEdit->setText(fileName);
}

void MainWindow::on_sendButton_clicked()
{
    QString contactName = ui->contactEdit->text();
    if (contactName.isEmpty()) {
        qWarning() << "Contact name is not entered";
        return;
    }

    const QComboBox *accountSelector = ui->accountSelector;
    int accountIndex = accountSelector->currentIndex();
    if (accountIndex == -1) {
        qWarning() << "Account for sending is not selected";
        return;
    }
    QString accountPath = accountSelector->itemData(accountIndex).toString();

    Tp::AccountPtr account = accountManager->accountForPath(accountPath);
    if (account.isNull()) {
        qWarning() << "Account path" << accountPath << "is invalid";
        return;
    }

    QString fileName = ui->fileEdit->text();
    if (fileName.isEmpty()) {
        qWarning() << "File name for sending is not specified";
        return;
    }

    QScopedPointer<QFile> file(new QFile(fileName));
    if (!file->open(QIODevice::ReadOnly)) {
        qWarning() << "File" << fileName << "cannot be opened";
        return;
    }

    // TODO: figure out the appropriate MIME type
    Tp::FileTransferChannelCreationProperties fileTransferProperties(
                fileName,
                QLatin1String("application/octet-stream"),
                file->size());

    Tp::PendingChannelRequest *channelRequest = account->createFileTransfer(
                contactName, fileTransferProperties);

    SendDialog *sendWindow = new SendDialog(this);
    sendWindow->setPendingChannelRequest(channelRequest);

    file->setParent(sendWindow);
    sendWindow->setFile(file.take());

    sendWindow->show();
    sendWindow->raise();
    sendWindow->activateWindow();
}
